from pdf2image import convert_from_path
from subprocess import call, DEVNULL
import os
import csv
import sys
import json

def read_json(jsonfile):
  data = []
  with open(jsonfile) as f:
    data = json.load(f)
  return data

def readCSV(filename):
  """Read csv file
  
  Args:
    filename (str): csv file name
  
  Returns:
    list of dict: records in csv file
  """
  if not os.path.isfile(filename):
    print("{} DOES NOT exist!!!".format(filename), file=sys.stderr)
    return []
  with open(filename) as csvfile:
    cursor = csv.reader(csvfile, skipinitialspace=True, delimiter='\t')
    header = next(cursor)
    return [dict(zip(header, row)) for row in cursor]

def pptx2pdf(pptx, pdffolder='./'):
  """Convert pptx into pdf

  Args:
    pptx (str): pptx file
    pdffile (str): folder that the pdf is stored in

  Returns:
    str: pdf file path 
  """
  if not os.path.isdir(pdffolder):
    os.mkdir(pdffolder) 
  call(['soffice', '--headless', '--convert-to', 'pdf', '--outdir', pdffolder, pptx], stdout=DEVNULL)
  pdfpath = os.path.join(pdffolder, os.path.basename(pptx).split('.')[0] + '.pdf')
  return pdfpath

def pdf2images(pdfpath, imgfolder='./', start=0, end=None):
  """Convert pdf into images

  Args:
    pdfpath (str): pdf file path
    imgfolder (str): image folder. Default is current folder
    start (int): start page of pdf
    end (int): end page pdf

  Returns:
    int: number of images stored
  """
  if not os.path.isdir(imgfolder):
    os.mkdir(imgfolder) 
  images = convert_from_path(pdfpath, thread_count=2, use_pdftocairo=True, size=(800, None), timeout=240)
  for index, image in enumerate(images[start:end]):
    image.save(os.path.join(imgfolder, str(index)+".jpg"))
  return len(images[start:end])





from pptx import Presentation
from linguappt.lib import read_json
import json
from abc import abstractmethod

class StructureKGPPT:
  """It is designed as an abstract class to be inheritated for phrase ppt generation of different languages,

  Note:
    **DON'T** use this class directly. To define a StructureKGPPT for a lanugage

      1. Define subclass inheriting ``StructureKGPPT``, e.g, ``ChineseStructureKGPPT`` 
      2. Define class variable ``_templates``, which is a :obj:`dict`, key is template genre, value is template path
      3. Define class varibale ``content_keys``, which is a :obj:`list`, containing the heads in csv file
      4. Implement method ``_create_structure_kg(self)``
 
  """

  def __init__(self, sourcefile, title="", genre="classic"):
    """Initialize ppt object, ppt title, and content from source csv file
    Args:
      sourcefile (str): csv file, whose content is written into ppt
      title (str): title written in ppt home slide
      genre (str): ppt template style 
    """

    if self.__class__.__name__ != "StructureKGPPT":
      self._assert_class_variables()
      self._template = self.__class__._templates[genre]
      self._prs = Presentation(self._template)
      self._title = title
      self._sourcefile = sourcefile
      self._assert_content()
    else:
      raise TypeError(self.__class__.__doc__)

  def _assert_class_variables(self):
    cls = self.__class__
    assert cls._templates != None
    assert isinstance(cls._templates, dict)
    assert cls.content_keys != None
    assert isinstance(cls.content_keys, list)


  def _assert_content(self):
    """Ensure the json file content has keys defined in ppt class
    """

    content = read_json(self._sourcefile) 
    for e in content:
      keys = e.keys()
      assert(len(keys) == len(self.__class__.content_keys))
      for k in keys:
        assert(k in self.__class__.content_keys)
    self.content = content


  def _create_opening(self):
    """Create home slide
    """

    layout = self._prs.slide_layouts.get_by_name("Opening")

    slide = self._prs.slides.add_slide(layout)
    holders = slide.shapes.placeholders
 
    title = holders[10] 
    title.text_frame.text = self._title
    subtitle = holders[11]
    subtitle.text_frame.text = '句型和知识点总结'

  def _create_ending(self):
    """Create ending slide
    """

    layout = self._prs.slide_layouts.get_by_name("Thanks")
    self._prs.slides.add_slide(layout)

  def _save_ppt(self, destfile):
    """Save ppt object into file
    """

    self._prs.save(destfile)

  def convert_to_ppt(self, destfile='test.pptx'):
    """Convert csv file containing vocabulary information into pptx file

    Args:
      destfile (str): pptx file path
    """

    self._create_opening()
    self._create_structure_kg()
    self._create_ending()

    self._save_ppt(destfile)    

  
  @abstractmethod
  def _create_structure_kg(self):
    """Create structure slide for each sentence
    """
    pass




from linguappt.phrase_ppt import PhrasePPT
import os
import json

class GermanPhrasePPT(PhrasePPT):
  """Create Phrase PPT for German study

  Attributes:
    content (list of dict): read from csv file
  """

  _template_dir = os.path.dirname(__file__)
  _templates = {
    "classic": os.path.join(_template_dir, 'templates/phrase_german_classic.pptx'),
  }
  lang = 'de'

  content_keys = ["sentence", "noun_phrases", "verbs"]

  def __init__(self, sourcefile, title="", genre="classic"):
    super().__init__(sourcefile, title, genre)

  def _create_phrase_and_verb(self, line):
    sentence_obj = json.loads(line["sentence"])
    ps_obj = json.loads(line["noun_phrases"])[:4]
    vs_obj = json.loads(line["verbs"])[:4]

    layout = self._prs.slide_layouts.get_by_name("Phrase and verb")
    slide = self._prs.slides.add_slide(layout)
    holders = slide.shapes.placeholders
  
    st_holder, sm_holder = holders[10], holders[11]
    st_holder.text_frame.text = sentence_obj["text"]
    sm_holder.text_frame.text = sentence_obj["meaning"]

    for index, p in enumerate(ps_obj):
      base_index = 12 + index*2

      t_holder, m_holder = holders[base_index], holders[base_index+1]
      t_holder.text_frame.text = p["text"]
      m_holder.text_frame.text = p["meaning"]

    for index, v in enumerate(vs_obj):
      base_index = 20+index*3
      verb_holder, origin_holder, form_holder = holders[base_index], holders[base_index+1], holders[base_index+2]

      verb_holder.text_frame.text = v["text"]
      origin_holder.text_frame.text = v["lemma"]
      form_holder.text_frame.text = v["form"].split(",")[1]
 
  def _create_phrase(self):
    """Create phrase by sentence, which are noun phrases, verb phrases
    """
    for line in self.content:
      self._create_phrase_and_verb(line)





from linguappt.structure_kg_ppt import StructureKGPPT
import os
import json

class GermanStructureKGPPT(StructureKGPPT):
  """Create StructureKG PPT for German study

  Attributes:
    content (list of dict): read from csv file
  """

  _template_dir = os.path.dirname(__file__)
  _templates = {
    "classic": os.path.join(_template_dir, 'templates/structure_kg_german_classic.pptx'),
  }
  lang = 'de'

  content_keys = ["sentence", "structure", "structure_rep", "kg"]

  def __init__(self, sourcefile, title="", genre="classic"):
    super().__init__(sourcefile, title, genre)

  def _create_structure(self, line):
    sentence_obj = line["sentence"]
    structure_str = line["structure_rep"]
    structure_obj = line["structure"]

    layout = self._prs.slide_layouts.get_by_name("Structure")
    slide = self._prs.slides.add_slide(layout)
    holders = slide.shapes.placeholders
  
    original_holder, translated_holder, structure_holder = holders[10], holders[11], holders[12]
    original_holder.text_frame.text = sentence_obj["text"]
    translated_holder.text_frame.text = sentence_obj["meaning"]
    structure_holder.text_frame.text = structure_str

    struct_analysis = ["{}    {}".format(p["text"], p["meaning"]) for p in structure_obj if p["explanation"]]
    struct_analysis_str = "\n".join(struct_analysis)
    struct_analysis_holder = holders[13]
    struct_analysis_holder.text_frame.text = struct_analysis_str 

  def _create_kg(self, line):
    sentence_obj = line["sentence"]
    kg_obj = line["kg"]

    layout = self._prs.slide_layouts.get_by_name("Detail")
    slide = self._prs.slides.add_slide(layout)
    holders = slide.shapes.placeholders
  
    original_holder, translated_holder = holders[10], holders[11]
    original_holder.text_frame.text = sentence_obj["text"]
    translated_holder.text_frame.text = sentence_obj["meaning"]

    kg = []
    for key, value in kg_obj.items():
      for v in value:
        kg.append("{}    {}".format(key, v["text"]))
    kg_str = "\n".join(kg)
    kg_holder = holders[12]
    kg_holder.text_frame.text = kg_str 


  def _create_structure_kg(self):
    """Create sturecture and kg by sentence
    """
    for line in self.content:
      self._create_structure(line)
      self._create_kg(line)





from linguappt import SpanishVocabPPT, EnglishVocabPPT
from linguappt import EnglishPhrasePPT, SpanishPhrasePPT, GermanPhrasePPT
from linguappt import EnglishStructureKGPPT, SpanishStructureKGPPT, GermanStructureKGPPT
from linguappt.lib import pptx2pdf, pdf2images 
from linguappt import __version__
import click
import json
import os
 
def _print_version(ctx, param, value):
  if not value or ctx.resilient_parsing:
      return
  click.echo(__version__)
  ctx.exit()
 
@click.command()
@click.option("--sourcecsv", prompt="source csv file path", help="Specify the source csv file path")
@click.option("--title", prompt="title of the pptx", help="Specify the title of the pptx")
@click.option("--lang", prompt="language", help="Specify the language")
@click.option("--destpptx", default="test.pptx", prompt="destination pptx file", help="Specify the destination pptx file name")
def vocabppt(sourcecsv, title, lang, destpptx):
  _PPTS = {
    "en": EnglishVocabPPT,
    "es": SpanishVocabPPT
  }

  _PPT = _PPTS[lang]

  phase = {"step": 1, "msg": "Start ppt generation"}
  print(json.dumps(phase))

  vp = _PPT(sourcecsv, title)
  vp.convert_to_ppt(destpptx)

  phase = {"step": 2, "msg": "Finish ppt generation"}
  print(json.dumps(phase))


@click.command()
@click.option("--sourcecsv", prompt="source csv file path", help="Specify the source csv file path")
@click.option("--title", prompt="title of the pptx", help="Specify the title of the pptx")
@click.option("--lang", prompt="language", help="Specify the language")
@click.option("--destpptx", default="test.pptx", prompt="destination pptx file", help="Specify the destination pptx file name")
def phraseppt(sourcecsv, title, lang, destpptx):
  _PPTS = {
    "en": EnglishPhrasePPT,
    "es": SpanishPhrasePPT,
    "de": GermanPhrasePPT
  }

  _PPT = _PPTS[lang]

  phase = {"step": 1, "msg": "Start ppt generation"}
  print(json.dumps(phase))

  vp = _PPT(sourcecsv, title)
  vp.convert_to_ppt(destpptx)

  phase = {"step": 2, "msg": "Finish ppt generation"}
  print(json.dumps(phase))

@click.command()
@click.option("--sourcejson", prompt="source json file path", help="Specify the source json file path")
@click.option("--title", prompt="title of the pptx", help="Specify the title of the pptx")
@click.option("--lang", prompt="language", help="Specify the language")
@click.option("--destpptx", default="test.pptx", prompt="destination pptx file", help="Specify the destination pptx file name")
def structurekgppt(sourcejson, title, lang, destpptx):
  _PPTS = {
    "en": EnglishStructureKGPPT,
    "es": SpanishStructureKGPPT,
    "de": GermanStructureKGPPT
  }

  _PPT = _PPTS[lang]

  phase = {"step": 1, "msg": "Start ppt generation"}
  print(json.dumps(phase))

  vp = _PPT(sourcejson, title)
  vp.convert_to_ppt(destpptx)

  phase = {"step": 2, "msg": "Finish ppt generation"}
  print(json.dumps(phase))




@click.command()
@click.option("--sourcepptx", prompt="source pptx file path", help="Sepcify the source pptx file path")
@click.option("--destdir", prompt="dest pdf and pictures directory", help="Sepcify the pdf and picture destionation directory")
def pptx2pdf2images(sourcepptx, destdir):
  phase = {"step": 1, "msg": "Start pdf generation"}
  print(json.dumps(phase))

  pdf = pptx2pdf(sourcepptx, destdir)

  phase = {"step": 2, "msg": "Finish pdf generation"}
  print(json.dumps(phase))

  phase = {"step": 3, "msg": "Start images generation"}
  print(json.dumps(phase))

  images_len = pdf2images(pdf, destdir)
   
  phase = {"step": 4, "msg": "Finish images generation", "images_len": images_len}
  print(json.dumps(phase))


@click.command()
@click.option('--version', is_flag=True, callback=_print_version, expose_value=False, is_eager=True)
@click.option('--ptype', prompt="parser type[VOCAB | PHRASE | SKG]", help="Specify the parse type, VOCAB or PHRASE or SKG")
@click.option("--sourcemeta", prompt="source meta file path", help="Specify the source meta file path")
@click.option("--lang", prompt="language", help="Specify the language")
@click.option("--name", default="test", prompt="output file name", help="Specify the file name")
@click.option("--pptxdir", prompt="dest pptx directory", help="Specify the pptx destination directory")
@click.option("--pdfdir", prompt="dest pdf directory", help="Sepcify the pdf destionation directory")
@click.option("--imgdir", prompt="dest image directory", help="Sepcify the preview image destionation directory")
def meta2media(ptype, sourcemeta, lang, name, pptxdir, pdfdir, imgdir):
  _PPTS = {
    "en_VOCAB": EnglishVocabPPT,
    "es_VOCAB": SpanishVocabPPT,
    "en_PHRASE": EnglishPhrasePPT,
    "es_PHRASE": SpanishPhrasePPT,
    "de_PHRASE": GermanPhrasePPT,
    "en_SKG": EnglishStructureKGPPT,
    "es_SKG": SpanishStructureKGPPT,
    "de_SKG": GermanStructureKGPPT,
  }

  _PPT = _PPTS[lang+"_"+ptype]
  phase = {"step": 1, "msg": "Start ppt generation"}
  print(json.dumps(phase), flush=True)

  if not os.path.isdir(pptxdir):
    os.mkdir(pptxdir) 

  vp = _PPT(sourcemeta, "歧舌AI备课助教")
  pptx = pptxdir + "/" + name +'.pptx'
  vp.convert_to_ppt(pptx)

  phase = {"step": 2, "msg": "Finish ppt generation, start pdf generation"}
  print(json.dumps(phase), flush=True)

  if not os.path.isdir(pdfdir):
    os.mkdir(pdfdir) 

  pdf = pptx2pdf(pptx, pdfdir)

  phase = {"step": 3, "msg": "Finish pdf generation, start images generation"}
  print(json.dumps(phase), flush=True)

  images_len = pdf2images(pdf, imgdir, 0, 6)
   
  phase = {"step": 4, "msg": "Finish images generation", "images_len": images_len}
  print(json.dumps(phase), flush=True)




from pptx import Presentation
from linguappt.lib import readCSV
import json
from abc import abstractmethod


class VocabPPT:
  """It is designed as an abstract class to be inheritated for vocabulary ppt generation of different languages,

  Note:
    **DON'T** use this class directly. To define a VocabPPT for a lanugage

      1. Define subclass inheriting ``VocabPPT``, e.g, ``ChineseVocabPPT`` 
      2. Define class variable ``_templates``, which is a :obj:`dict`, key is template genre, value is template path
      3. Define class variable ``content_keys``, which is a :obj:`list`, containing the heads in csv file
      4. Implement methods ``_create_noun(self, v)``, ``_create_verb(self, v)``, ``_create_adj(self, v)``, where ``v`` is a vocab object, corresponding to a record (one line) in csv file
    
  """

  def __init__(self, sourcefile, title="", genre="classic"):
    """Initialize ppt object, ppt title, and content from source csv file

    Args:
      sourcefile (str): csv file, whose content is written into ppt
      title (str): title written in ppt home slide
      genre (str): ppt template style 
    """

    if self.__class__.__name__ != "VocabPPT":
      self._assert_class_variables()
      self._template = self.__class__._templates[genre]
      self._prs = Presentation(self._template)
      self._title = title
      self._sourcefile = sourcefile
      self._assert_content()
      self._divide_by_pos()
    else:
      raise TypeError(self.__class__.__doc__)

  def _assert_class_variables(self):
    cls = self.__class__
    assert cls._templates != None
    assert isinstance(cls._templates, dict)
    assert cls.content_keys != None
    assert isinstance(cls.content_keys, list)

  def _divide_by_pos(self):
    """Store words in ``self.word_distribution`` as dict, whose key is PoS (part of speech) defined in ``EnglisgVocabMeta.pos_info`` and value is a list of words.
    """

    word_distribution  = {}

    for e in self.content:
      key = self.__class__._metainfo.get_pos(e["dict_pos"])
      if key in self.__class__.ALLOWED_POSES:
        if key not in word_distribution:
          word_distribution[key] = []
        word_distribution[key].append(e)
    self.word_distribution = word_distribution


  def _assert_content(self):
    """Ensure the csv file content has keys defined in ppt class
    """

    content = readCSV(self._sourcefile) 
    for e in content:
      keys = e.keys()
      assert(len(keys) == len(self.__class__.content_keys))
      for k in keys:
        assert(k in self.__class__.content_keys)
    self.content = content

  def _create_opening(self):
    """Create home slide
    """

    layout = self._prs.slide_layouts.get_by_name("Opening")
    slide = self._prs.slides.add_slide(layout)
    holders = slide.shapes.placeholders
 
    title = holders[10] 
    title.text_frame.text = self._title
    subtitle = holders[11]
    subtitle.text_frame.text = '词汇总结'

  def _create_statistics(self):
    """Create statistic slide
    """

    layout = self._prs.slide_layouts.get_by_name("Statistics")
    slide = self._prs.slides.add_slide(layout)
    holders = slide.shapes.placeholders

    for index, (pos, ws) in enumerate(self.word_distribution.items()):
      pos_name = holders[11+2*index]
      num = holders[10+2*index]
      pos_name.text_frame.text = self.__class__._metainfo.get_pos_name(pos).upper() 
      num.text_frame.text = str(len(ws))

  def _create_ending(self):
    """Create ending slide
    """

    layout = self._prs.slide_layouts.get_by_name("Thanks")
    self._prs.slides.add_slide(layout)

  def _create_vocab_title(self, title_content, subtitle_content):
    """Create vocabulary title slide for PoS

    Args:
      title_content (str): title displayed in slide
      subtitle_content (str): subtitle displayed in slide
    """

    layout = self._prs.slide_layouts.get_by_name('Title for pos')
    slide = self._prs.slides.add_slide(layout)
    holders = slide.shapes.placeholders

    title = holders[10]
    title.text_frame.text = title_content
    subtitle = holders[11]
    subtitle.text_frame.text = subtitle_content

  def _create_default_word(self, v):
    """Create default vocabulary slide, displaying word and its meaning

    Args:
      v (dict): vocabuary object, its keys should be the same as defined in subclass variable ``content_keys``
    """

    layout = self._prs.slide_layouts.get_by_name("Default")
    slide = self._prs.slides.add_slide(layout)
    holders = slide.shapes.placeholders

    pos, noun, meaning = holders[10], holders[11], holders[12]
    pos.text_frame.text = v["dict_pos"]
    noun.text_frame.text = v["word"]
    ms = v["meaning"].split(",")
    if len(ms) > 4:
      ms = ms[:4]
    meaning.text_frame.text = "\n".join(ms) 

    note = slide.notes_slide
    note.notes_text_frame.text = v["word"]

  def _create_default_word_with_examples(self, v):
    """Create default vocabulary with examples slide, displaying word, its meaning and examples 

    Args:
      v (dict): vocabuary object, its keys should be the same as defined in subclass variable ``content_keys``
    """

    layout = self._prs.slide_layouts.get_by_name("Default with examples")
    slide = self._prs.slides.add_slide(layout)
    holders = slide.shapes.placeholders

    pos, noun, meaning = holders[10], holders[11], holders[12]
    pos.text_frame.text = v["dict_pos"]
    noun.text_frame.text = v["word"]
    ms = v["meaning"].split(",")
    if len(ms) > 4:
      ms = ms[:4]
    meaning.text_frame.text = "\n".join(ms) 

    examples = json.loads(v["examples"])
    if len(examples) >= 1:
      original, translated = holders[13], holders[14]
      ex = examples[0]
      original.text_frame.text = ex["original"]
      translated.text_frame.text = ex["translated"]
    if len(examples) >= 2:
      original, translated = holders[15], holders[16]
      ex = examples[1]
      original.text_frame.text = ex["original"]
      translated.text_frame.text = ex["translated"]

    note = slide.notes_slide
    note.notes_text_frame.text = v["word"]


  def _create_vocab_group(self, pos, vocabs):
    """Create slides for vocabularies according to PoS

    Args:
      pos (str): PoS of vocabularies
      vocabs (list of dict): vocabularies with the same PoS
    """

    for v in vocabs:
      if pos == "noun":
        self._create_noun(v) 
      elif pos == "adj":
        self._create_adj(v)
      elif pos == "verb":
        self._create_verb(v)
      #else:
      #  pass


  def _create_vocab(self):
    """Create vocab group slides, which are noun, adj, verb, etc, restrained by subclass variable ALLOWED_POSES
    """

    for pos, ws in self.word_distribution.items():
      if len(ws) > 0:
        subtitle = self.__class__._metainfo.get_pos_cn_name(pos)
        title = self.__class__._metainfo.get_pos_cn_name(pos).upper()
        self._create_vocab_title(title, subtitle)
        self._create_vocab_group(pos, ws)

  def _save_ppt(self, destfile):
    """Save ppt object into file
    """

    self._prs.save(destfile)

  def convert_to_ppt(self, destfile='test.pptx'):
    """Convert csv file containing vocabulary information into pptx file

    Args:
      destfile (str): pptx file path
    """

    self._create_opening()
    self._create_statistics()
    self._create_vocab()
    self._create_ending()

    self._save_ppt(destfile)    

  @abstractmethod
  def _create_noun(self, v):
    """Create slide for vocabulary with PoS as noun

    Args:
      v (dict): vocabuary object, its keys should be the same as defined in subclass variable ``content_keys``
    """
    pass

  @abstractmethod
  def _create_verb(self, v):
    """Create slide for vocabulary with PoS as verb 

    Args:
      v (dict): vocabuary object, its keys should be the same as defined in subclass variable ``content_keys``
    """
    pass

  @abstractmethod
  def _create_adj(self, v):
    """Create slide for vocabulary with PoS as adjective 

    Args:
      v (dict): vocabuary object, its keys should be the same as defined in subclass variable ``content_keys``
    """
    pass




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['linguappt',
 'linguappt._entry',
 'linguappt.de',
 'linguappt.en',
 'linguappt.es']

package_data = \
{'': ['*'],
 'linguappt.de': ['templates/*'],
 'linguappt.en': ['templates/*'],
 'linguappt.es': ['templates/*']}

install_requires = \
['click>=7.1.2,<8.0.0',
 'pdf2image>=1.12.1,<2.0.0',
 'python-pptx>=0.6.18,<0.7.0']

entry_points = \
{'console_scripts': ['lingua_meta2media = linguappt._entry.command:meta2media',
                     'lingua_phraseppt = linguappt._entry.command:phraseppt',
                     'lingua_pptx2pdf2images = '
                     'linguappt._entry.command:pptx2pdf2images',
                     'lingua_pptx_validate = '
                     'linguappt._entry.validation:validate',
                     'lingua_structurekgppt = '
                     'linguappt._entry.command:structurekgppt',
                     'lingua_vocabppt = linguappt._entry.command:vocabppt']}

setup_kwargs = {
    'name': 'linguappt',
    'version': '0.1.21',
    'description': 'PPT generator for language learning',
    'long_description': '# Installation\n\n```\npip3 install --verbose linguappt \n```\n\n# Usage\n\nPlease refer to [api docs](https://qishe-nlp.github.io/linguappt/).\n\n### Execute usage\n\n* Validate ppt template\n```\nlingua_pptx_validate --pptx [pptx file]\n```\n\n* Convert vocabulary csv file into ppt file\n```\nlingua_vocabppt --sourcecsv [vocab csv file] --lang [language] --title [title shown in ppt] --destpptx [pptx file]\n```\n\n* Convert phrase csv file into ppt file\n```\nlingua_phraseppt --sourcecsv [phrase csv file] --lang [language] --title [title shown in ppt] --destpptx [pptx file]\n```\n\n\n* Convert ppt into pdf\n```\nlingua_pptx2pdf --sourcepptx [pptx file] --destdir [dest directory storing pdf and images]\n```\n\n### Package usage\n```\nfrom linguappt import SpanishVocabPPT, EnglishVocabPPT\nfrom linguappt import EnglishPhrasePPT, SpanishPhrasePPT\n\ndef vocabppt(sourcecsv, title, lang, destpptx):\n  _PPTS = {\n    "en": EnglishVocabPPT,\n    "es": SpanishVocabPPT\n  }\n\n  _PPT = _PPTS[lang]\n\n  vp = _PPT(sourcecsv, title)\n  vp.convert_to_ppt(destpptx)\n\ndef phraseppt(sourcecsv, title, lang, destpptx):\n  _PPTS = {\n    "en": EnglishPhrasePPT,\n    "es": SpanishPhrasePPT\n  }\n\n  _PPT = _PPTS[lang]\n\n  vp = _PPT(sourcecsv, title)\n  vp.convert_to_ppt(destpptx)\n\n\n```\n\n# Development\n\n### Clone project\n```\ngit clone https://github.com/qishe-nlp/linguappt.git\n```\n\n### Install [poetry](https://python-poetry.org/docs/)\n\n### Install dependencies\n```\npoetry update\n```\n\n### Test\n```\npoetry run pytest -rP --capture=sys\n```\nwhich run tests under `tests/*`\n\n\n### Execute\n```\npoetry run lingua_pptx_validate --help\npoetry run lingua_vocabppt --help\npoetry run lingua_phraseppt --help\n\npoetry run lingua_pptx2pdf2images --help\npoetry run lingua_csv2media --help\n```\n\n### Create sphinx docs\n```\npoetry shell\ncd apidocs\nsphinx-apidoc -f -o source ../linguappt\nmake html\npython -m http.server -d build/html\n```\n\n### Host docs on github pages\n```\ncp -rf apidocs/build/html/* docs/\n```\n\n### Build\n* Change `version` in `pyproject.toml` and `linguappt/__init__.py`\n* Build python package by `poetry build`\n\n### Git commit and push\n\n### Publish from local dev env\n* Set pypi test environment variables in poetry, refer to [poetry doc](https://python-poetry.org/docs/repositories/)\n* Publish to pypi test by `poetry publish -r test`\n\n### Publish through CI \n* Github action build and publish package to [test pypi repo](https://test.pypi.org/)\n\n```\ngit tag [x.x.x]\ngit push origin master\n```\n\n* Manually publish to [pypi repo](https://pypi.org/) through [github action](https://github.com/qishe-nlp/linguappt/actions/workflows/pypi.yml)\n\n',
    'author': 'Phoenix Grey',
    'author_email': 'phoenix.grey0108@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/qishe-nlp/linguappt',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from pptx import Presentation
from pptx.util import Inches, Pt
from pptx.dml.color import RGBColor

import os
import click

# TODO : may need to be updated
def _test_template_usage():
  template_to_be_tested = './linguappt/es/templates/vocab_spanish_classic.pptx'
  prs = Presentation(template_to_be_tested)
  templates = prs.slide_layouts
  
  # template idx:0 - name:Title and subtitle
  layout = templates.get_by_name("Title and subtitle") 
  slide = prs.slides.add_slide(layout)
  holders = slide.shapes.placeholders


  title = holders[10] 
  title.text_frame.text = 'Extra E1-1'
  subtitle = holders[11]
  subtitle.text_frame.text = '解析'

  # template idx:2 - name:Title 1
  layout = templates.get_by_name("Title 1")
  slide = prs.slides.add_slide(layout)
  holders = slide.shapes.placeholders


  title = holders[10]
  title.text_frame.text = '名词'
  subtitle = holders[11]
  subtitle.text_frame.text = 'El nombre'
  note = slide.notes_slide
  note.notes_text_frame.text = "xxxxxxxxxxx" 


  # template 2
  slide = prs.slides.add_slide(templates[2])
  holders = slide.shapes.placeholders


  word = holders[10]
  explaination = holders[11]
  word.text_frame.text = 'casa'
  explaination.text_frame.text = '家'

  # template 3
  slide = prs.slides.add_slide(templates[3])
  holders = slide.shapes.placeholders

  word = holders[10]
  explaination = holders[11]
  word.text_frame.text = 'niña'
  word.text_frame.paragraphs[0].font.color.rgb = RGBColor(0, 0, 255)
  explaination.text_frame.text = '小女孩'


  # template 4
  slide = prs.slides.add_slide(templates[4])
  holders = slide.shapes.placeholders


  for index in range(9):
    word = holders[10+2*index]
    explaination = holders[11+2*index]
    word.text_frame.text = str(index)
    explaination.text_frame.text = str(index) 

  # template 5
  slide = prs.slides.add_slide(templates[13])
  holders = slide.shapes.placeholders

  for e in holders:
    print('%d %s' % (e.placeholder_format.idx, e.name))
 
  for index in range(4):
    word = holders[10+2*index]
    explaination = holders[11+2*index]
    word.text_frame.text = str(index*10)
    explaination.text_frame.text = str(index*10) 

  prs.save('test.pptx')

def _default():
  prs = Presentation()
  blank_slide_layout = prs.slide_layouts[6]
  slide = prs.slides.add_slide(blank_slide_layout)

  left = top = width = height = Inches(1)
  txBox = slide.shapes.add_textbox(left, top, width, height)
  tf = txBox.text_frame

  tf.text = "This is text inside a textbox"

  p = tf.add_paragraph()
  p.text = "This is a second paragraph that's bold"
  p.font.bold = True

  p = tf.add_paragraph()
  p.text = "This is a third paragraph that's big"
  p.font.size = Pt(40)

  prs.save('test.pptx')

